///////////////////////////////////////////////////////////////////////////////
// main.cpp
// ========
// main driver to test SetWindowsHookEx() to monitor keyboard/mouse events
//
//  AUTHOR: Song Ho Ahn (song.ahn@gmail.com)
// CREATED: 2023-04-12
// UPDATED: 2023-04-19
///////////////////////////////////////////////////////////////////////////////

#define WIN32_LEAN_AND_MEAN             // exclude rarely-used stuff from Windows headers

#include <windows.h>
#include <commctrl.h>                   // common controls
#include "Window.h"
#include "DialogWindow.h"
#include "ControllerMain.h"
#include "ControllerForm.h"
#include "resource.h"
#include "Log.h"


// function declarations
int mainMessageLoop(HACCEL hAccelTable=0);



///////////////////////////////////////////////////////////////////////////////
// main function of a windows application
///////////////////////////////////////////////////////////////////////////////
int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR cmdArgs, int cmdShow)
{
    // create view & controllers
    Win::ViewForm viewForm;
    Win::ControllerMain ctrlMain;
    Win::ControllerForm ctrlForm(&viewForm);

    // create main window
    Win::Window winMain(hInst, L"Test Event Hooks", 0, &ctrlMain);
    winMain.setWidth(500);
    winMain.setHeight(400);
    winMain.setWindowStyleEx(WS_EX_WINDOWEDGE);
    winMain.create();
    Win::log("Main Window is created.");

    Win::DialogWindow dialog(hInst, IDD_FORMVIEW, winMain.getHandle(), &ctrlForm);
    dialog.create();
    Win::log("Form Dialog is created.");

    // get size of dialog
    RECT rect;
    ::GetWindowRect(dialog.getHandle(), &rect);

    // re-size main window same as the dialog
    // it also includes non-client area, such as border, titlebar, menu,
    // invisible border around window, etc.
    DWORD style = (DWORD)::GetWindowLongPtr(winMain.getHandle(), GWL_STYLE);
    DWORD styleEx = (DWORD)::GetWindowLongPtr(winMain.getHandle(), GWL_EXSTYLE);
    ::AdjustWindowRectEx(&rect, style, FALSE, styleEx);
    ::SetWindowPos(winMain.getHandle(), 0, 0, 0, (rect.right-rect.left), (rect.bottom-rect.top), SWP_NOZORDER);

    // show all windows
    dialog.show();
    winMain.show();

    // install low-level keyboard/mouse event hooks
    // and pass view ptr to display event messages to it
    // you must uninstall the kooks before closing the window (see ControllerMain::destroy())
    ctrlMain.initEventHooks(&viewForm);

    // main message loop //////////////////////////////////////////////////////
    int exitCode;
    HACCEL hAccelTable = 0;
    //hAccelTable = ::LoadAccelerators(hInst, MAKEINTRESOURCE(ID_ACCEL));
    exitCode = mainMessageLoop(hAccelTable);

    Win::log("Application terminated.");
    return exitCode;
}



///////////////////////////////////////////////////////////////////////////////
// main message loop
///////////////////////////////////////////////////////////////////////////////
int mainMessageLoop(HACCEL hAccelTable)
{
    HWND activeHandle;
    MSG msg;

    while(::GetMessage(&msg, 0, 0, 0) > 0)  // loop until WM_QUIT(0) received
    {
        // determine the activated window is dialog box
        // skip if messages are for the dialog windows
        activeHandle = GetActiveWindow();
        if(::GetWindowLongPtr(activeHandle, GWL_EXSTYLE) & WS_EX_CONTROLPARENT) // WS_EX_CONTROLPARENT is automatically added by CreateDialogBox()
        {
            if(::IsDialogMessage(activeHandle, &msg))
                continue;   // message handled, back to while-loop
        }

        // now, handle window messages
        if(!::TranslateAccelerator(msg.hwnd, hAccelTable, &msg))
        {
            ::TranslateMessage(&msg);
            ::DispatchMessage(&msg);
        }
    }

    return (int)msg.wParam;                 // return nExitCode of PostQuitMessage()
}
